@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -25,6 +25,9 @@ These are for Python2 only constructs, which will give SyntaxError on
 other versions.
 """
 
+# While we use that for comparison code, no need to compile that.
+# nuitka-project: --nofollow-import-to=nuitka
+
 import os
 import sys
 
@@ -38,9 +41,7 @@ sys.path.insert(
 
 # isort:start
 
-from nuitka.tools.testing.Common import checkDebugPython, executeReferenceChecked
-
-checkDebugPython()
+from nuitka.tools.testing.Common import executeReferenceChecked
 
 x = 17
 
